@@ -8,10 +8,20 @@
 
 import warnings
 
-from peewee import (AutoField, BigAutoField, BigIntegerField,
-                    CharField, DeferredThroughModel, DoubleField,
-                    FixedCharField, FloatField, ForeignKeyField,
-                    IntegerField, ManyToManyField, TextField)
+from peewee import (
+    AutoField,
+    BigAutoField,
+    BigIntegerField,
+    CharField,
+    DeferredThroughModel,
+    DoubleField,
+    FixedCharField,
+    FloatField,
+    ForeignKeyField,
+    IntegerField,
+    ManyToManyField,
+    TextField
+)
 from playhouse.postgres_ext import ArrayField
 
 from sdssdb.exceptions import SdssdbUserWarning
@@ -20,6 +30,19 @@ from .. import BaseModel
 from . import database
 
 
+# When adding a foreign key like below to a peewee model class
+#
+# gaia_xmatch = ForeignKeyField(Gaia_edr3_sdssdr13_best_neighbour,
+#                               field='original_ext_source_id',
+#                               column_name='objid',
+#                               backref='+')
+#
+# one needs to add the below line to the peewee model class
+# for the remote table Gaia_edr3_sdssdr13_best_neighbour
+#
+# original_ext_source_id = TextField()
+
+
 class CatalogdbModel(BaseModel):
 
     class Meta:
@@ -77,6 +100,7 @@ class AllWise(CatalogdbModel):
 
     cntr = BigIntegerField(primary_key=True)
     designation = TextField()
+    tmass_key = BigIntegerField()
 
     class Meta:
         table_name = 'allwise'
@@ -122,6 +146,38 @@ class EROSITASupersetCompactobjects(CatalogdbModel):
         table_name = 'erosita_superset_compactobjects'
 
 
+class EROSITASupersetv1AGN(CatalogdbModel):
+
+    pkey = BigIntegerField(primary_key=True)
+
+    class Meta:
+        table_name = 'erosita_superset_v1_agn'
+
+
+class EROSITASupersetv1Clusters(CatalogdbModel):
+
+    pkey = BigIntegerField(primary_key=True)
+
+    class Meta:
+        table_name = 'erosita_superset_v1_clusters'
+
+
+class EROSITASupersetv1Stars(CatalogdbModel):
+
+    pkey = BigIntegerField(primary_key=True)
+
+    class Meta:
+        table_name = 'erosita_superset_v1_stars'
+
+
+class EROSITASupersetv1Compactobjects(CatalogdbModel):
+
+    pkey = BigIntegerField(primary_key=True)
+
+    class Meta:
+        table_name = 'erosita_superset_v1_compactobjects'
+
+
 class ElbadryRix(CatalogdbModel):
 
     pkey = BigIntegerField(primary_key=True)
@@ -135,6 +191,11 @@ class TwoMassPSC(CatalogdbModel):
     pts_key = IntegerField(primary_key=True)
     designation = TextField()
 
+    allwise = ForeignKeyField(AllWise,
+                              field='tmass_key',
+                              column_name='pts_key',
+                              backref='+')
+
     class Meta:
         table_name = 'twomass_psc'
 
@@ -178,6 +239,14 @@ class Gaia_DR3(CatalogdbModel):
 class Gaia_DR2_Neighbourhood(CatalogdbModel):
 
     pkey = BigIntegerField(primary_key=True)
+    dr2_source_id = ForeignKeyField(Gaia_DR2,
+                                    field='source_id',
+                                    column_name='dr2_source_id',
+                                    backref='+')
+    dr3_source_id = ForeignKeyField(Gaia_DR3,
+                                    field='source_id',
+                                    column_name='dr3_source_id',
+                                    backref='+')
 
     class Meta:
         table_name = 'gaia_dr2_neighbourhood'
@@ -194,12 +263,22 @@ class Gedr3spur_main(CatalogdbModel):
 class Gaia_edr3_allwise_best_neighbour(CatalogdbModel):
 
     source_id = BigIntegerField(primary_key=True)
+    original_ext_source_id = TextField()
+
+    gaia_dr3 = ForeignKeyField(Gaia_DR3,
+                               field='source_id',
+                               column_name='source_id',
+                               backref='+')
+    external = ForeignKeyField(AllWise,
+                               field='designation',
+                               column_name='original_ext_source_id',
+                               backref='+')
 
     class Meta:
         table_name = 'gaia_edr3_allwise_best_neighbour'
 
 
-class Gaia_edr3_gaia_edr3_allwise_neighbourhood(CatalogdbModel):
+class Gaia_edr3_allwise_neighbourhood(CatalogdbModel):
 
     class Meta:
         table_name = 'gaia_edr3_allwise_neighbourhood'
@@ -208,26 +287,67 @@ class Gaia_edr3_gaia_edr3_allwise_neighbourhood(CatalogdbModel):
 class Gaia_edr3_panstarrs1_best_neighbour(CatalogdbModel):
 
     source_id = BigIntegerField(primary_key=True)
+    original_ext_source_id = TextField()
+
+    gaia_dr3 = ForeignKeyField(Gaia_DR3,
+                               field='source_id',
+                               column_name='source_id',
+                               backref='+')
 
     class Meta:
         table_name = 'gaia_edr3_panstarrs1_best_neighbour'
 
 
-class Gaia_edr3_gaia_edr3_panstarrs1_neighbourhood(CatalogdbModel):
+class Gaia_edr3_panstarrs1_neighbourhood(CatalogdbModel):
 
     class Meta:
         table_name = 'gaia_edr3_panstarrs1_neighbourhood'
 
 
+# Note this is Gaia_dr3 and not Gaia_edr3
+class Gaia_dr3_ravedr6_best_neighbour(CatalogdbModel):
+
+    source_id = BigIntegerField(primary_key=True)
+    original_ext_source_id = TextField()
+
+    gaia_dr3 = ForeignKeyField(Gaia_DR3,
+                               field='source_id',
+                               column_name='source_id',
+                               backref='+')
+
+    class Meta:
+        table_name = 'gaia_dr3_ravedr6_best_neighbour'
+
+
+class Gaia_dr3_ravedr6_neighbourhood(CatalogdbModel):
+
+    source_id = BigIntegerField(primary_key=True)
+    original_ext_source_id = TextField()
+
+    gaia_dr3 = ForeignKeyField(Gaia_DR3,
+                               field='source_id',
+                               column_name='source_id',
+                               backref='+')
+
+    class Meta:
+        table_name = 'gaia_dr3_ravedr6_neighbourhood'
+
+
 class Gaia_edr3_sdssdr13_best_neighbour(CatalogdbModel):
 
     source_id = BigIntegerField(primary_key=True)
+    original_ext_source_id = TextField()
+
+    gaia_dr3 = ForeignKeyField(Gaia_DR3,
+                               field='source_id',
+                               column_name='source_id',
+                               backref='+')
 
     class Meta:
         table_name = 'gaia_edr3_sdssdr13_best_neighbour'
 
 
-class Gaia_edr3_gaia_edr3_sdssdr13_neighbourhood(CatalogdbModel):
+class Gaia_edr3_sdssdr13_neighbourhood(CatalogdbModel):
 
     class Meta:
         table_name = 'gaia_edr3_sdssdr13_neighbourhood'
@@ -241,7 +361,7 @@ class Gaia_edr3_skymapperdr2_best_neighbour(CatalogdbModel):
         table_name = 'gaia_edr3_skymapperdr2_best_neighbour'
 
 
-class Gaia_edr3_gaia_edr3_skymapperdr2_neighbourhood(CatalogdbModel):
+class Gaia_edr3_skymapperdr2_neighbourhood(CatalogdbModel):
 
     class Meta:
         table_name = 'gaia_edr3_skymapperdr2_neighbourhood'
@@ -261,12 +381,20 @@ class Gaia_edr3_tmass_psc_xsc_best_neighbour(CatalogdbModel):
 class Gaia_edr3_tmass_psc_xsc_best_neighbour2(CatalogdbModel):
 
     source_id = BigIntegerField(primary_key=True)
+    gaia_dr3 = ForeignKeyField(Gaia_DR3,
+                               field='source_id',
+                               column_name='source_id',
+                               backref='+')
+    external = ForeignKeyField(TwoMassPSC,
+                               field='designation',
+                               column_name='original_ext_source_id',
+                               backref='+')
 
     class Meta:
         table_name = 'gaia_edr3_tmass_psc_xsc_best_neighbour2'
 
 
-class Gaia_edr3_gaia_edr3_tmass_psc_xsc_neighbourhood(CatalogdbModel):
+class Gaia_edr3_tmass_psc_xsc_neighbourhood(CatalogdbModel):
 
     class Meta:
         table_name = 'gaia_edr3_tmass_psc_xsc_neighbourhood'
@@ -286,12 +414,18 @@ class Gaia_edr3_tycho2tdsc_merge_best_neighbour(CatalogdbModel):
 class Gaia_edr3_tycho2tdsc_merge_best_neighbour2(CatalogdbModel):
 
     source_id = BigIntegerField(primary_key=True)
+    original_ext_source_id = TextField()
+
+    gaia_dr3 = ForeignKeyField(Gaia_DR3,
+                               field='source_id',
+                               column_name='source_id',
+                               backref='+')
 
     class Meta:
         table_name = 'gaia_edr3_tycho2tdsc_merge_best_neighbour2'
 
 
-class Gaia_edr3_gaia_edr3_tycho2tdsc_merge_neighbourhood(CatalogdbModel):
+class Gaia_edr3_tycho2tdsc_merge_neighbourhood(CatalogdbModel):
 
     class Meta:
         table_name = 'gaia_edr3_tycho2tdsc_merge_neighbourhood'
@@ -383,6 +517,14 @@ class Gaia_dr3_vari_agn(CatalogdbModel):
         table_name = 'gaia_dr3_vari_agn'
 
 
+class Gaia_dr3_vari_rrlyrae(CatalogdbModel):
+
+    source_id = BigIntegerField(primary_key=True)
+
+    class Meta:
+        table_name = 'gaia_dr3_vari_rrlyrae'
+
+
 class Gaia_dr3_nss_two_body_orbit(CatalogdbModel):
 
     # There are duplicate source_id in the table
@@ -418,6 +560,95 @@ class GUVCat(CatalogdbModel):
         table_name = 'guvcat'
 
 
+class Galah_dr3(CatalogdbModel):
+
+    sobject_id = BigIntegerField(primary_key=True)
+
+    dr2_source_id = ForeignKeyField(
+        model=Gaia_DR2,  # remote model
+        field='source_id',  # remote column name
+        column_name='dr2_source_id',  # local column name
+        backref='+')  # '+' means do not create backref
+
+    dr3_source_id = ForeignKeyField(
+        model=Gaia_DR3,  # remote model
+        field='source_id',  # remote column name
+        column_name='dr3_source_id',  # local column name
+        backref='+')  # '+' means do not create backref
+
+    class Meta:
+        table_name = 'galah_dr3'
+
+
+class Visual_binary_gaia_dr3(CatalogdbModel):
+
+    source_id1 = BigIntegerField(primary_key=True)
+
+    dr2_source_id1 = ForeignKeyField(
+        model=Gaia_DR2,  # remote model
+        field='source_id',  # remote column name
+        column_name='dr2_source_id1',  # local column name
+        backref='+')  # '+' means do not create backref
+
+    dr2_source_id2 = ForeignKeyField(
+        model=Gaia_DR2,  # remote model
+        field='source_id',  # remote column name
+        column_name='dr2_source_id2',  # local column name
+        backref='+')  # '+' means do not create backref
+
+    # The LHS variable name dr3_source_id1
+    # is different from the RHS local column_name
+    # because source_id1 is the primary key above.
+    dr3_source_id1 = ForeignKeyField(
+        model=Gaia_DR3,  # remote model
+        field='source_id',  # remote column name
+        column_name='source_id1',  # local column name
+        backref='+')  # '+' means do not create backref
+
+    dr3_source_id2 = ForeignKeyField(
+        model=Gaia_DR3,  # remote model
+        field='source_id',  # remote column name
+        column_name='source_id2',  # local column name
+        backref='+')  # '+' means do not create backref
+
+    class Meta:
+        table_name = 'visual_binary_gaia_dr3'
+
+
+class Lamost_dr6(CatalogdbModel):
+
+    # There are duplicate source_id
+    # in the table lamost_dr6.
+    # Hence, source_id is not a primary key
+    # for this table.
+
+    dr3_source_id = ForeignKeyField(
+        model=Gaia_DR3,  # remote model
+        field='source_id',  # remote column name
+        column_name='source_id',  # local column name
+        backref='+')  # '+' means do not create backref
+
+    class Meta:
+        table_name = 'lamost_dr6'
+
+
+class WD_gaia_dr3(CatalogdbModel):
+
+    gaiaedr3 = BigIntegerField(primary_key=True)
+
+    # source_id for Gaia edr3 and Gaia dr3 is the same.
+    # So below we have foreign key from local column gaiaedr3 to
+    # remote column gaia_dr3_source(source_id).
+    dr3_source_id = ForeignKeyField(
+        model=Gaia_DR3,  # remote model
+        field='source_id',  # remote column name
+        column_name='gaiaedr3',  # local column name
+        backref='+')  # '+' means do not create backref
+
+    class Meta:
+        table_name = 'wd_gaia_dr3'
+
+
 class KeplerInput_DR10(CatalogdbModel):
 
     kic_kepler_id = BigIntegerField(primary_key=True)
@@ -672,6 +903,7 @@ class SDSS_DR17_APOGEE_Allvisits(CatalogdbModel):
 class SDSS_DR19p_Speclite(CatalogdbModel):
 
     pk = BigIntegerField(primary_key=True)
+    bestobjid = BigIntegerField()
 
     class Meta:
         table_name = 'sdss_dr19p_speclite'
@@ -689,6 +921,10 @@ class Tycho2(CatalogdbModel):
 
     designation = TextField(primary_key=True)
     tycid = IntegerField(null=False)
+    gaia_xmatch = ForeignKeyField(Gaia_edr3_tycho2tdsc_merge_best_neighbour2,
+                                  field='original_ext_source_id',
+                                  column_name='designation2',
+                                  backref='+')
 
     class Meta:
         table_name = 'tycho2'
@@ -795,6 +1031,20 @@ class Zari18ums(CatalogdbModel):
         table_name = 'zari18ums'
 
 
+class Xpfeh_gaia_dr3(CatalogdbModel):
+
+    source_id = BigIntegerField(primary_key=True)
+
+    gaia_dr3 = ForeignKeyField(
+        model=Gaia_DR3,
+        field='source_id',
+        column_name='source_id',
+        backref='+')
+
+    class Meta:
+        table_name = 'xpfeh_gaia_dr3'
+
+
 class Legacy_Survey_DR8(CatalogdbModel):
 
     ls_id = BigIntegerField(primary_key=True)
@@ -831,6 +1081,25 @@ class Legacy_Survey_DR10a(CatalogdbModel):
         table_name = 'legacy_survey_dr10a'
 
 
+class Legacy_Survey_DR10(CatalogdbModel):
+
+    ls_id = BigIntegerField(primary_key=True)
+    ref_cat = TextField()
+    ref_id = BigIntegerField()
+
+    gaia2 = ForeignKeyField(Gaia_DR2,
+                            field='source_id',
+                            column_name='gaia_dr2_source_id',
+                            backref='+')
+    gaia3 = ForeignKeyField(Gaia_DR3,
+                            field='source_id',
+                            column_name='gaia_dr3_source_id',
+                            backref='+')
+
+    class Meta:
+        table_name = 'legacy_survey_dr10'
+
+
 class eBOSS_Target_v5(CatalogdbModel):
 
     sdss = ForeignKeyField(SDSS_DR13_PhotoObj,
@@ -996,10 +1265,13 @@ class BHM_CSC_v2(CatalogdbModel):
     class Meta:
         table_name = 'bhm_csc_v2'
 
+# note that BHM_CSC_v3 is below Panstarrs1 to allow foreign key linking
+
 
 class Gaia_DR2_WD_SDSS(CatalogdbModel):
 
     pk = BigIntegerField(primary_key=True)
+    wd = TextField()
 
     @property
     def specobj(self):
@@ -1061,6 +1333,11 @@ class Gaia_DR2_WD(CatalogdbModel):
                           object_id_name='source_id',
                           backref='+')
 
+    wd = ForeignKeyField(Gaia_DR2_WD_SDSS,
+                         field='wd',
+                         column_name='wd',
+                         backref='+')
+
     @property
     def sdss(self):
         """Returns records from `.Gaia_DR2_WD_SDSS` with matching ``wd``."""
@@ -1324,11 +1601,38 @@ class PS1_g18(CatalogdbModel):
 class Panstarrs1(CatalogdbModel):
 
     catid_objid = BigIntegerField(primary_key=True)
+    extid_hi_lo = ForeignKeyField(Gaia_edr3_panstarrs1_best_neighbour,
+                                  field='original_ext_source_id',
+                                  column_name='extid_hi_lo',
+                                  backref='+')
 
     class Meta:
         table_name = 'panstarrs1'
 
 
+class BHM_CSC_v3(CatalogdbModel):
+
+    pk = BigIntegerField(primary_key=True)
+
+    lsdr10 = ForeignKeyField(Legacy_Survey_DR10,
+                             field='ls_id',
+                             column_name='ls_dr10_lsid',
+                             backref='+')
+
+    gaia3 = ForeignKeyField(Gaia_DR3,
+                            field='source_id',
+                            column_name='gaia_dr3_srcid',
+                            backref='+')
+
+    twomass = ForeignKeyField(TwoMassPSC,
+                              field='designation',
+                              column_name='tmass_designation',
+                              backref='+')
+
+    class Meta:
+        table_name = 'bhm_csc_v3'
+
+
 class GLIMPSE(CatalogdbModel):
 
     pk = BigIntegerField(primary_key=True)
@@ -1520,6 +1824,30 @@ class XMM_OM_SUSS_4_1(CatalogdbModel):
         table_name = 'xmm_om_suss_4_1'
 
 
+class XMM_OM_SUSS_5_0(CatalogdbModel):
+
+    pk = BigIntegerField(primary_key=True)
+
+    class Meta:
+        table_name = 'xmm_om_suss_5_0'
+
+
+class HECATE_1_1(CatalogdbModel):
+
+    pgc = IntegerField(primary_key=True)
+
+    class Meta:
+        table_name = 'hecate_1_1'
+
+
+class MILLIQUAS_7_7(CatalogdbModel):
+
+    pk = BigIntegerField(primary_key=True)
+
+    class Meta:
+        table_name = 'milliquas_7_7'
+
+
 class GeometricDistances_Gaia_DR2(CatalogdbModel):
 
     source_id = BigIntegerField(primary_key=True)
@@ -1568,6 +1896,39 @@ class BHM_RM_v0(CatalogdbModel):
         table_name = 'bhm_rm_v0'
 
 
+class BHM_RM_v1(CatalogdbModel):
+
+    pk = BigIntegerField(primary_key=True)
+
+    lsdr8 = ForeignKeyField(Legacy_Survey_DR8,
+                            field='ls_id',
+                            column_name='ls_id_dr8',
+                            backref='+')
+
+    lsdr10 = ForeignKeyField(Legacy_Survey_DR10,
+                             field='ls_id',
+                             column_name='ls_id_dr10',
+                             backref='+')
+
+    gaia2 = ForeignKeyField(Gaia_DR2,
+                            field='source_id',
+                            column_name='gaia_dr2_source_id',
+                            backref='+')
+
+    gaia3 = ForeignKeyField(Gaia_DR3,
+                            field='source_id',
+                            column_name='gaia_dr3_source_id',
+                            backref='+')
+
+    ps1 = ForeignKeyField(Panstarrs1,
+                          field='catid_objid',
+                          column_name='panstarrs1_catid_objid',
+                          backref='+')
+
+    class Meta:
+        table_name = 'bhm_rm_v1'
+
+
 class BHM_RM_v0_2(BHM_RM_v0):
     pass
 
@@ -1615,6 +1976,21 @@ class SDSS_DR13_PhotoObj_Primary(CatalogdbModel):
                           object_id_name='objid',
                           backref='+')
 
+    tic_ext = ForeignKeyField(TIC_v8_Extended,
+                              field='sdss',
+                              column_name='objid',
+                              backref='+')
+
+    sdss19p = ForeignKeyField(SDSS_DR19p_Speclite,
+                              field='bestobjid',
+                              column_name='objid',
+                              backref='+')
+
+    gaia_xmatch = ForeignKeyField(Gaia_edr3_sdssdr13_best_neighbour,
+                                  field='original_ext_source_id',
+                                  column_name='objid',
+                                  backref='+')
+
     class Meta:
         table_name = 'sdss_dr13_photoobj_primary'
 
